#include <set>
#include <string>
#include <vector>
#include <map>
#include <utility>
#include "itemset.h"
#include "auxiliary.h"

using namespace std;




class AttrClusterer
{

public:
	AttrClusterer();
	~AttrClusterer();
	
	bool set_data_filename(string filename);
	bool read_data();
	triplet<double> input_descrlen(string filename);
	bool save_clustering(string outfilename);
	
	void set_verbosity(int v);
	void set_scoretype(int st);
	void set_searchstrategy(int ss);
	
	void set_param_k(int k);
	void set_param_stop(bool stop);
	void set_param_beam_width(int w);
	void set_param_lambda(double lambda);
	void set_param_iterations(int iter);
	void set_param_cycles(int cycles);
	
	void hierarchical_agglomerative();
	void simulated_annealing();
	void beam_search();
	
	double canonical_descrlen();
	
private:
	void   init_clusters();
	void   init_clusters_random(bool quiet=false);
	void   cleanup_clusters();
	double cluster_merge_gain(int i, int j, int n=0);
	void   merge_clusters(int i, int j, int n=0);
	void   split_cluster(int i, const Attrset & aset1, const Attrset & aset2, int n=0);
	
	void   transactions_to_binary();
	
	double partition_descr_len();
	pair<double,double> cluster_descr_len(int i, int n=0);
	pair<double,double> singleton_descr_len(int attr);
	pair<double,double> attrset_descr_len(int i, int j, int n=0);
	pair<double,double> attrset_descr_len(const Attrset & aset1, const Attrset & aset2);
	triplet<double> clustering_score(int n=0);
	
	void print_overall_best_clustering();
	
public:
	enum SCORETYPES {SCO_ENTROPY=0, SCO_TWOPART=1, SCO_SEMIREFINED=2, SCO_REFINED=3, SCO_BIC=4, SCO_AIC=5, SCO_COUNTDOWN=6, SCO_TEST=7};
	enum SEARCHSTRATEGIES {SRCH_INPUT=0, SRCH_HIERARCHICAL=1, SRCH_ANNEAL=2, SRCH_BEAM=3};
	enum DATATYPES {DATA_NONE=0, DATA_BINARY=1, DATA_CATEGORICAL=2};
	enum FORMATS {FMT_PLAINTXT=0, FMT_XML=2};
	
	string datafilename;
	string scorefilename;
	
	int scoretype;
	int searchstrategy;
	int datatype;
	int resultformat;
	
private:
	vector<Itemset>   transactions;
	vector<int*>      binary_transactions;
	int               trans_count;
	set<int>          attributes;
	set<int>          items;
	int               attr_count;
	int               item_count;
	double          * item_freqs;
	int               smallest_attr;
	
	map<pair<int,string>, int > attrval_2_item;
	map<int, set<int> >         attr_items;
	
	int    param_k;
	bool   param_stop;
	int    param_beam_width;
	double param_lambda;
	int    param_iterations;
	int    param_cycles;
	
	map<int, Attrset>              * clusters;							// cluster ID ; cluster elements (i.e., attributes)
	map<int, pair<double,double> > * cluster_descr_length;				// cluster ID ; two-part description length
	map<int, Attrset>                overall_best_clustering;
	triplet<double>                  overall_best_clustering_score;
	
	double log_bell;
	
	int running_time; // in seconds
	
	static map<int, string> scoretypenames;
	static map<int, string> searchstrategynames;
	
	int verbosity;
	
public:
	static const string name;
	static const string version;
	static const string url;
	static const string author;
	
};
