#include <algorithm>
#include <math.h>
#include <vector>
#include <stdlib.h>
#include <string>
#include <sstream>
#include <utility>

#ifndef _AUX_H
#define _AUX_H 


inline
double log_2(double x)
{
	return log(x) / log(2.0);
}


template <class T> inline
double log_stirling(T k)
{
	// log of stirling approximation of factorial
	return 0.5 * (1.0 + log_2(3.141592653589793) + log_2(k)) + k * (log_2(k) - 1.0/log(2.0)) + (1.0/(12.0*k) / log(2.0));
}


template <class T> inline
double log_factorial(T k)
{
	if (k > 500) {
		return log_stirling(k);
	}
	else {
		double v = 0.0;
		for (T i=T(2); i<=k; ++i)
			v += log_2(i);
		return v;
	}
}


//template<class T> inline
//double log_gamma_function(T k)
//{
//	const double fractional = k - floor(k);
//	if (fractional == 0.0)
//		return log_factorial(k-1);
//	else if (fractional == 0.5)
//		return log_factorial(2*k) - log_factorial(k) - 2*k + 0.5 * log_2(3.141592653589793);
//	else {
//		
//	}
//}


inline
double log_binomial(const int n, int k)
{
	if (k > n/2)
		k = n - k;
	
	double c = 0.0;
	for (int i=0; i<k; ++i) {
		c += log_2(n - i);
		c -= log_2(i + 1);
	}
	
	return c;
}


// compute the expression log_2(2^a + 2^b)
inline
double log_sum_log(double a, double b)
{
	if (a > b)
		std::swap(a, b);
	
	return log_2(pow(2, b-a) + 1.0) + a;
}


inline
double log_bell_number(int n)
{
	if (n == 0)
		return 0.0;
	
	double * t0 = new double[n];
	double * t1 = new double[n];
	
	t0[0] = 0.0;
	
	for (int i=1; i<n; ++i) {
		t1[0] = t0[i-1];
		for (int j=0; j<i; ++j)
			t1[j+1] = log_sum_log(t0[j], t1[j]);
		std::swap(t0, t1);
	}
	
	double result = t0[n-1];
	
	delete [] t0;
	delete [] t1;
	
	return result;
}


// [0,1)
inline
double randfloat()
{
	return (double) rand() / ((double)RAND_MAX + 1.0);
}


// [a,b)
inline
int randint(int a, int b)
{
	return rand() % (b-a) + a;
}


inline
std::string int_2_string(int integer)
{
	std::stringstream ss;
	ss << integer;
	return ss.str();
}


inline
std::string double_2_string(double real)
{
	std::stringstream ss;
	ss.precision(10);
	ss.setf(std::ios::floatfield);
	ss << real;
	return ss.str();
}


template <class T> class mypair : public std::pair<T,T>
{
public:
	mypair<T>(const T & t1, const T & t2) : std::pair<T,T>(t1, t2) { }
	
	mypair<T> & operator=(const mypair<T> & arg) { std::pair<T,T>::operator=(arg); }
	
	mypair<T> & operator+=(const mypair<T> & arg) {
		this->first += arg.first;
		this->second += arg.second;
		return *this;
	}
	
};


template <class T> struct triplet
{
	triplet() : first(T(0)), second(T(0)), third(T(0)) {}
	triplet(T f, T s, T t) : first(f), second(s), third(t) {}
	
	bool operator<(const triplet<T> & other) const {
		if (first < other.first)
			return true;
		else if (first == other.first) {
			if (second < other.second)
				return true;
			else if (second == other.second)
				return third < other.second;
		}
		return false;
	}
	
	T first;
	T second;
	T third;
};


#endif
