#include "auxiliary.h"
#include <utility>

using std::pair;

/*
 Note: this is actually more of an int-vector map, but for its intended purpose this is much faster
 */

class BitVectorMap
{
public:
	BitVectorMap(int);
	~BitVectorMap();
	
	void increment(int *);
	mypair<double> get_entropy_and_log_sum();
	double get_refined_entropy(double domsize);
	double map_sum(double (*function)(double));
	int get_size() const { return mapsize; }
	
private:
	void extend_mapping_array();
	
	int    vsize;
	pair<int*, int> * mapping;
	int mapsize;
	int allocated;
	
	double sum;
	
};



BitVectorMap::BitVectorMap(int vs) : vsize(vs), mapsize(0), allocated(64), sum(0)
{
	mapping = new pair<int*, int>[allocated];
	
	return;
}



BitVectorMap::~BitVectorMap()
{
	for (int i=0; i<mapsize; ++i)
		delete [] mapping[i].first;

	delete [] mapping;
	
	return;
}



void BitVectorMap::increment(int * bitvector)
{
	int lo = 0, hi = mapsize, mid;
	
	while (lo < hi) {
		mid = (lo+hi) >> 1; // lo + ((hi-lo) >> 1);
		int ix = 0;
		while (ix < vsize && mapping[mid].first[ix] == bitvector[ix]) ++ix;
		if (ix == vsize) {
			++(mapping[mid].second);
			++sum;
			return;
		}
		else if (mapping[mid].first[ix] < bitvector[ix])
			lo = mid + 1;
		else
			hi = mid;
	}
	
	if (mapsize == allocated)
		extend_mapping_array();
	for (int i=mapsize; i>lo; --i)
		mapping[i] = mapping[i-1];
	int * b = new int[vsize];
	memcpy(b, bitvector, vsize * sizeof(int));
	mapping[lo] = pair<int*, int>(b, 1);
	++mapsize;
	
	++sum;
	
	return;
}



inline
void BitVectorMap::extend_mapping_array()
{
	allocated += 64;
	pair<int*, int> * newmapping = new pair<int*, int>[allocated];
	
	memcpy(newmapping, mapping, mapsize * sizeof(pair<int*, int>));
	
	delete [] mapping;
	mapping = newmapping;
	
	return;
}



mypair<double> BitVectorMap::get_entropy_and_log_sum()
{
	mypair<double> hls = mypair<double>(0.0, 0.0);
	for (int i=0; i<mapsize; ++i) {
		double f = mapping[i].second / sum;
		hls.first -= f * log_2(f);
		hls.second -= log_2(f);
	}
	
	return hls;
}



double BitVectorMap::get_refined_entropy(double domsize)
{
	double re;
	
	if (0.5*domsize == floor(0.5*domsize))
		re = log_factorial(sum + 0.5*domsize - 1) - log_factorial(0.5*domsize - 1);
	else
		re = log_factorial(2*sum + domsize-1.0) - log_factorial(sum + floor(0.5*domsize)) - 2*sum - log_factorial(domsize-1.0) + log_factorial(floor(0.5*domsize));
	
	for (int i=0; i<mapsize; ++i) {
		re -= log_factorial(2 * mapping[i].second) - log_factorial(mapping[i].second) - 2 * mapping[i].second;
	}
	
	return re;
}



double BitVectorMap::map_sum(double (*function)(double))
{
	double mapsum = 0.0;
	
	for (int i=0; i<mapsize; ++i) {
		mapsum += function(mapping[i].second);
	}
	
	return mapsum;
}
