#include "attributecluster.h"
#include "acoptions.h"
#include <iostream>
#include <string>
#include "stdio.h"

using std::cerr;
using std::string;


int main (int argc, char * argv[])
{	
	printf("%s, %s (code by %s)\n%s\n\n", AttrClusterer::name.c_str(), AttrClusterer::version.c_str(), AttrClusterer::author.c_str(), AttrClusterer::url.c_str());
	
	ACOptions options;
	
	if (argc < 2 || argv[1] == string("-h") || argv[1] == string("--help")) {
		options.print_usage(cerr);
		return 1;
	}
	else {
		int parse_status = options.parse_options(argc, argv);
		if (parse_status != 0)
			return 1;
	}
	
	AttrClusterer ac;
	ac.set_verbosity(options.verbosity);
	ac.set_scoretype(options.scoretype);
	ac.set_searchstrategy(options.searchstrategy);
	if (options.partitionfilename.size())
		ac.set_searchstrategy(AttrClusterer::SRCH_INPUT);
	
	// set the algorithm parameters
	ac.set_param_k(options.k);
	ac.set_param_stop(options.stop);
	ac.set_param_beam_width(options.beamwidth);
	ac.set_param_lambda(options.lambda);
	ac.set_param_iterations(options.iterations);
	ac.set_param_cycles(options.cycles);
	
	if (options.scores.size())
		ac.scorefilename = options.scores;
	
	// read the data
	bool rd = ac.set_data_filename(options.filename) && ac.read_data();
	if (rd == false)
		return 1;
	
	// run algorithm
	switch (ac.searchstrategy)
	{
		case AttrClusterer::SRCH_INPUT:
			ac.input_descrlen(options.partitionfilename);
			break;
		case AttrClusterer::SRCH_HIERARCHICAL:
			ac.hierarchical_agglomerative();
			break;
		case AttrClusterer::SRCH_ANNEAL:
			ac.simulated_annealing();
			break;
		case AttrClusterer::SRCH_BEAM:
			ac.beam_search();
			break;
		default:
			break;
	}
	
	// store the results
	if (options.output.size())
		ac.save_clustering(options.output);
	
	return 0;
	
}
