/*
 * intmap.h
 *
 *  Created on: Jun 9, 2009
 *      Author: michael
 */

#ifndef INTMAP_H_
#define INTMAP_H_

#include <utility>
#include <memory.h>

using namespace std;


template <class T> class IntMap {
public:
	IntMap();
	~IntMap();
	void insert(int key, T val);
	T get(int key);
	pair<int, T> get_r(int key, int rr);

	pair<int, T> *map;
	int size;

private:
	int capacity;

	static const int block_size = 4;
	static const int not_found = 0;

};



template <class T> inline IntMap<T>::IntMap() : size(0), capacity(block_size) {
	map = new pair<int,T>[capacity];
}


template <class T> inline IntMap<T>::~IntMap(){
	delete [] map;
}


template <class T> inline void IntMap<T>::insert(int key, T val) {
	if (size==capacity) {
		capacity += block_size;
		pair<int,T> *newmap = new pair<int,T>[capacity];
		memcpy(newmap, map, size * sizeof(pair<int,T>));
		delete [] map;
		map = newmap;
	}
	
	if (size==0) {
		map[0] = pair<int,T>(key,val);
		++size;
		return;
	}
	
	int l = 0, r = size - 1, c = 0;
	while (l <= r) {
		c = (l + r) >> 1;
		if (map[c].first > key) {
			l = c + 1;
		}
		else {
			r = c - 1;
		}
	}
	if (map[c].first > key)
		++c;
	for (int i=size; i>c; --i) {
		map[i] = map[i-1];
	}
	map[c] = pair<int, T>(key,val);
	++size;
	
	return;
}


template <class T> inline T IntMap<T>::get(int key) {
	int l = 0, r = size - 1, c;
	while (l <= r) {
		c = (l + r) >> 1;
		if (map[c].first == key) {
			return map[c].second;
		}
		else if (map[c].first > key) {
			l = c + 1;
		}
		else {
			r = c - 1;
		}
	}
	return T(not_found);
}


#endif /* INTMAP_H_ */
