#include "itemset.h"
#include <algorithm>
#include <vector>
#include <iostream>

using namespace std;


#ifndef _ITEMSET_H
#define  _ITEMSET_H


Itemset::Itemset(int item) : size(1)
{
	iset = new int[size];
	iset[0] = item;
	return;
}

Itemset::Itemset(int size, bool dummy) : size(size)
{
	iset = new int[size];

	return;
}

Itemset::Itemset(const Itemset & iset1, const Itemset & iset2) : size(iset1.size + iset2.size)
{
	iset = new int[size];
	int *k = iset;
	int *i1 = iset1.iset;
	int *i2 = iset2.iset;
	const int *i1end = i1 + iset1.size;
	const int *i2end = i2 + iset2.size;
	while (i1 < i1end && i2 < i2end) {
		if (*i1 < *i2)
			*k++ = *i1++;
		else
			*k++ = *i2++;
	}
	if (i1 < i1end)
		while (i1 < i1end)
			*k++ = *i1++;
	else if (i2 < i2end)
		while (i2 < i2end)
			*k++ = *i2++;
	
	return;
}

void Itemset::intersect_into(const Itemset & iset1, const Itemset & iset2)
{
	size = 0;
	int *i1 = iset1.iset;
	int *i2 = iset2.iset;
	const int *i1end = i1 + iset1.size;
	const int *i2end = i2 + iset2.size;
	while (i1 < i1end && i2 < i2end) {
		if (*i1 < *i2)
			++i1;
		else if (*i1 > *i2)
			++i2;
		else {
			iset[size++] = *i1;
			++i1; ++i2;
		}
	}
	
	return;
}

Itemset::Itemset(const vector<int> & arg_iset) : size(arg_iset.size())
{
	iset = new int[size];
	for (int i=0; i<size; ++i)
		iset[i] = arg_iset[i];
	return;
}

Itemset::Itemset(const Itemset & itemset) : size(itemset.size)
{
	iset = new int[size];
	copy(itemset.iset, itemset.iset+size, iset);
	return;
}

Itemset::~Itemset()
{
	delete [] iset;
}

int& Itemset::operator[](int i)
{
	return iset[i];
}


bool Itemset::operator==(const Itemset & other) const
{
	if (size != other.size)
		return false;
	
	for (int i=0; i<size; ++i)
		if (iset[i] != other.iset[i])
			return false;
	
	return true;
}


bool Itemset::contains(int item) const
{
	if (size == 0)
		return false;
	
	int lo = 0, hi = size, mid = 0;
	while (lo < hi) {
		mid = lo + ((hi-lo) >> 1);
		if (iset[mid] < item)
			lo = mid + 1;
		else if (iset[mid] > item)
			hi = mid;
		else
			return true;
	}
	
	return iset[mid] == item;
}

#endif
